package com.sd.business.service.order.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderInfoSelectDto;
import com.sd.business.entity.order.dto.OrderPackageBomDto;
import com.sd.business.entity.order.dto.OrderSkuDto;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.order.po.OrderSkuProductionCost;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.OrderPackageBomVo;
import com.sd.business.entity.order.vo.OrderSkuBomVo;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.order.OrderInfoMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.OrderInfoService;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuProductionCostService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import com.sd.framework.util.sql.Where;
import com.sd.mq.config.ArtworkConfig;
import com.sd.mq.entity.TempArtworkMessage;
import com.sd.mq.util.RabbitMqUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class OrderInfoServiceImpl extends ServiceImpl<OrderInfoMapper, OrderInfo> implements OrderInfoService {

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private OrderSkuProductionCostService orderSkuProductionCostService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysRoleService roleService;

    @Override
    public Page<OrderInfoVo> getPage(OrderInfoSelectDto dto) {

        Where<OrderInfoVo> sql = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(Department::getName, OrderInfoVo::getDepartmentName)

                .from(OrderInfo.class)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)

                .orderByDesc("if(t1.status = 60, 1, 0)")
                .orderByDesc(OrderInfo::getId)
                .eq(OrderInfo::getStatus, dto.getStatus())
                .eq(OrderInfo::getSettlementStatus, dto.getSettlementStatus())
                .eq(OrderInfo::getFlowStatus, dto.getFlowStatus())
                .like(OrderInfo::getCode, dto.getCode())
                .like(OrderInfo::getWlnCode, dto.getWlnCode())
                .ge(OrderInfo::getWlnCreateTime, dto.getBeginTime())
                .le(OrderInfo::getWlnCreateTime, dto.getEndTime())
                .findInSet(OrderInfo::getExceptionType, dto.getExceptionType())
                .eq(Department::getId, dto.getDepartmentId())
                .like(Department::getName, dto.getDepartmentName());

        List<Long> detpIdList = sysUserService.getDetpIdListByUserId(SecurityUtils.getUserId());
        if (!detpIdList.isEmpty()) {
            sql.in(OrderInfo::getDepartmentId, detpIdList);
            sql.eq(OrderInfo::getCreateUser, SecurityUtils.getUserId());
        }

        if (StatusConstant.YES.equals(dto.getLinkedStatementOfAccount())) {
            sql.isNotNull(OrderInfo::getStatementOfAccountId);
            sql.in(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey(), OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        } else if (StatusConstant.NO.equals(dto.getLinkedStatementOfAccount())) {
            sql.isNull(OrderInfo::getStatementOfAccountId);
            sql.in(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey(), OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        }

        if (StatusConstant.YES.equals(dto.getException())) {
            sql.ne(OrderInfo::getExceptionType, StatusConstant.NO);
        } else if (StatusConstant.NO.equals(dto.getException())) {
            sql.eq(OrderInfo::getExceptionType, StatusConstant.NO);
        }

        Page<OrderInfoVo> page = sql.page(dto);
        List<OrderInfoVo> records = page.getRecords();

        if (records.isEmpty()) {
            return page;
        }

        for (OrderInfoVo record : records) {
            // 设置异常提示
            String exceptionType = record.getExceptionType();
            if (exceptionType.equals("0")) {
                continue;
            }
            String exceptionTypeDetail = Arrays.stream(exceptionType.split(","))
                    .map(Convert::toInt)
                    .map(OrderExceptionTypeEnum::getEnumValue)
                    .collect(Collectors.joining(","));
            record.setExceptionTypeDetail(exceptionTypeDetail);
        }

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {

        OrderInfoVo vo = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(Department::getName, OrderInfoVo::getDepartmentName)
                .from(OrderInfo.class)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)
                .eq(OrderInfo::getId, id)
                .one();

        Assert.notNull(vo, "未知订单");

        // 订单sku列表
        List<OrderSkuVo> orderSkuList = Sql.create(OrderSkuVo.class)
                .selectAll(OrderSku.class)
                .selectAs(SkuSpec::getCode, OrderSkuVo::getSkuSpecCode)
                .selectAs(SkuSpec::getName, OrderSkuVo::getSkuSpecName)
                .selectAs(BomSpec::getCode, OrderSkuVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderSkuVo::getBomSpecName)
                .from(OrderSku.class)
                .leftJoin(SkuSpec.class, OrderSku::getSkuSpecId)
                .leftJoin(BomSpec.class, OrderSku::getBomSpecId)
                .eq(OrderSku::getOrderId, id)
                .list();

        // 包材配件/单品
        List<OrderSkuBomVo> orderSkuBomList = Sql.create(OrderSkuBomVo.class)
                .selectAll(OrderSkuBom.class)
                .selectAs(BomSpec::getCode, OrderSkuBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderSkuBomVo::getBomSpecName)
                .selectAs(Bom::getBomClassifyId, OrderSkuBomVo::getBomClassifyId)
                .from(OrderSkuBom.class)
                .leftJoin(BomSpec.class, OrderSkuBom::getBomSpecId)
                .leftJoin(Bom.class, BomSpec::getBomId)
                .eq(OrderSkuBom::getOrderId, id)
                .list();

        // 包装明细
        List<OrderPackageBomVo> orderPackageBomList = Sql.create(OrderPackageBomVo.class)
                .selectAll(OrderPackageBom.class)
                .selectAs(BomSpec::getCode, OrderPackageBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderPackageBomVo::getBomSpecName)
                .from(OrderPackageBom.class)
                .leftJoin(BomSpec.class, OrderPackageBom::getBomSpecId)
                .eq(OrderPackageBom::getOrderId, id)
                .list();

        // 赋值
        Map<Long, List<OrderSkuBomVo>> map = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBomVo::getOrderSkuId));
        orderSkuList.forEach(item -> item.setOrderSkuBomList(map.getOrDefault(item.getId(), Collections.emptyList())));
        vo.setOrderSkuList(orderSkuList);
        vo.setOrderPackageBomList(orderPackageBomList);

        return vo;
    }

    @DSTransactional
    @Override
    public void add(OrderInfoDto dto) {
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        if (ObjectUtil.equal(dto.getStatus(), OrderStatusEnum.STOCK_PREPARATION.getKey())
                && ObjectUtil.notEqual(dto.getType(), 2)) {
            // 先判断成品是否能锁定
            if (inventoryFinishedService.lockNoSourceStorage(orderSkuList)) {
                dto.setStockType(StatusConstant.YES);
            } else {
                lockStorage(orderSkuList);
                dto.setStockType(StatusConstant.NO);
            }
            dto.setLockStorage(StatusConstant.YES);
        } else {
            dto.setLockStorage(StatusConstant.NO);
        }

        // 新增订单
        dto.setSource(1);
        dto.setCode("PI" + new Date().getTime());
        dto.setSettlementStatus(1);
        dto.setTaxRate(new BigDecimal("10.00"));
        dto.setTag("0");
        dto.setExceptionType(OrderExceptionTypeEnum.NORMAL.getKey().toString());
        if (Objects.equals(dto.getType(), 2)) {
            dto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());
        } else if (Objects.equals(dto.getType(), 1)) {
            // 拥有采购角色的账号创建的自主订单为采购订单
            Long userId = SecurityUtils.getUserId();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }
        // 委外订单修改金额
        if (ObjectUtil.equals(dto.getType(), 2)) {
            dto.setProductTotalAmount(BigDecimal.ZERO);
            dto.setDeliveryMaterialsFee(BigDecimal.ZERO);
            dto.setManagementFee(BigDecimal.ZERO);
            dto.setPackingLabor(dto.getPackingLabor().multiply(new BigDecimal(2)));
            dto.setTotalAmount(dto.getProductTotalAmount()
                    .add(dto.getCustomProcessingFee())
                    .add(dto.getLssueFee())
                    .add(dto.getDeliveryMaterialsFee())
                    .add(dto.getPackingLabor())
                    .add(dto.getPackagingMaterialCost())
                    .add(dto.getManagementFee()));
        }
        save(dto);

        // 新增订单产品包材配件
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .peek(item -> {
                    // 委外订单修改金额
                    if (ObjectUtil.equals(dto.getType(), 2)) {
                        item.setUnitPrice(BigDecimal.ZERO);
                        item.setLssueFee(BigDecimal.ZERO);
                        item.setDeliveryMaterialsFee(BigDecimal.ZERO);
                        item.setManagementFee(BigDecimal.ZERO);
                        item.setPackingLabor(item.getPackingLabor().multiply(new BigDecimal(2)));
                    }
                })
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.saveBatch(tempOrderSkuList);

        // 新增订单产品包装信息
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.saveBatch(orderSkuBomList);

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.getOrDefault(item.getId(), new ArrayList<>());
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee()
                    .multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(
                    orderSkuProductionCost.getMaterialCost()
                            .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                            .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                            .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.saveBatch(orderSkuProductionCostList);

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.notEqual(dto.getStatus(), OrderStatusEnum.DRAFT.getKey())
                && ObjectUtil.equal(dto.getFlowStatus(), FlowStatusEnum.PASS.getKey())
                && ObjectUtil.equals(dto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            dto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.addByOrderId(dto.getId());
            update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, dto.getId()));
        }

        // 保存订单包装信息
        if (ObjectUtil.isNotEmpty(dto.getOrderPackageBomList())) {
            List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
            List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                    .peek(item -> item.setOrderId(dto.getId()))
                    .map(item -> (OrderPackageBom) item)
                    .collect(Collectors.toList());
            orderPackageBomService.saveBatch(orderPackageBomList);
        }

        // 保存产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.saveFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 保存附件
        ObsFileUtil.saveFile(dto.getFileList(), dto.getId());
    }

    @DSTransactional
    @Override
    public void edit(OrderInfoDto dto) {
        // 订单为委外订单时，订单分类为委外订单
        if (Objects.equals(dto.getType(), 2)) {
            dto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());
        } else if (Objects.equals(dto.getType(), 1)) {
            // 拥有采购角色的账号创建的自主订单为采购订单
            Long userId = dto.getCreateUser();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }

        // 更新订单
        updateById(dto);

        // 更新订单产品
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, dto.getId());

        // 更新订单产品包材配件
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, dto.getId());

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.getOrDefault(item.getId(), new ArrayList<>());
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee()
                    .multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(
                    orderSkuProductionCost.getMaterialCost()
                            .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                            .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                            .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.editLinked(orderSkuProductionCostList, OrderSkuProductionCost::getOrderId, dto.getId());

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.equals(dto.getStatus(), OrderStatusEnum.STOCK_PREPARATION.getKey())
                && ObjectUtil.equals(dto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            dto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.addByOrderId(dto.getId());
            update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, dto.getId()));
        }

        // 更新订单产品包装信息
        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
        if (tempOrderPackageBomList == null) {
            tempOrderPackageBomList = new ArrayList<>();
        }
        List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderPackageBom) item)
                .collect(Collectors.toList());
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, dto.getId());

        // 更新订单产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.editFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 更新订单附件
        ObsFileUtil.editFile(dto.getFileList(), dto.getId());
    }

    @DSTransactional
    @Override
    public void delete(Long id) {
        removeById(id);
        orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
        orderSkuBomService.remove(q -> q.eq(OrderSkuBom::getOrderId, id));
        orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, id));
        orderSkuProductionCostService.remove(q -> q.eq(OrderSkuProductionCost::getOrderId, id));
        productionWorkOrderService.remove(q -> q.eq(ProductionWorkOrder::getOrderId, id));

        ObsFileUtil.removeFile(id);
    }

    @DSTransactional
    @Override
    public synchronized void deleteAndStore(Long id) {
        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        // 解锁库存
        unLockStorage(orderInfo);

        if (orderInfo.getStatus() <= OrderStatusEnum.STOCK_PREPARATION.getKey()) {
            delete(id);
            return;
        }

        // 获取订单商品主材和包材
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));

        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        // 合并主材和包材
        List<InOutStorageBom> inOutStorageBomList = orderSkuList.stream().map(item -> {
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(item.getBomSpecId());
            inOutStorageBom.setQuantity(item.getQuantity());
            return inOutStorageBom;
        }).collect(Collectors.toList());

        for (OrderSkuBom orderSkuBom : orderSkuBomList) {
            OrderSku orderSku = orderSkuMap.get(orderSkuBom.getOrderSkuId());
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(orderSkuBom.getBomSpecId());
            inOutStorageBom.setQuantity(orderSkuBom.getQuantity().multiply(orderSku.getQuantity()));
            inOutStorageBomList.add(inOutStorageBom);
        }

        // 合并数量
        Map<Long, InOutStorageBom> map = inOutStorageBomList.stream().collect(Collectors.toMap(
                InOutStorageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }
        ));
        inOutStorageBomList = new ArrayList<>(map.values());

        // 区分半成品仓和包材仓库
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(map.keySet());

        // 主材从半成品仓入库
        semiFinishedProductOutStorage(inOutStorageBomList, bomSpecBoMap, orderInfo);

        // 包材从包材仓出库
        packagingMaterialOutStorage(inOutStorageBomList, bomSpecBoMap, orderInfo);

        // 删除订单回滚成品仓库存
        inventoryFinishedService.removeOrder(id);

        delete(id);
    }

    @DSTransactional
    @Override
    public void deleteValidated(Long id) {
        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        if (Objects.equals(orderInfo.getDelFlag(), StatusConstant.NOT_DELETED)
                && orderInfo.getStatus() > OrderStatusEnum.STOCK_PREPARATION.getKey()) {
            throw new ServiceException("订单正在生产中，无法直接删除，需要走审批流程");
        }

        unLockStorage(orderInfo);

        // 删除订单回滚成品仓库存
        inventoryFinishedService.removeOrder(id);

        // 删除订单以及订单关联数据
        delete(id);
    }

    @DSTransactional
    @Override
    public void confirmation(OrderInfoDto dto) {
        OrderInfo order = getById(dto.getId());

        if (ObjectUtil.notEqual(order.getStatus(), OrderStatusEnum.DRAFT.getKey())
                && ObjectUtil.notEqual(order.getStatus(), OrderStatusEnum.UNDER_REVIEW.getKey())) {
            throw new ServiceException("订单不在草稿或待确认状态");
        }

        // 自主订单需要锁定库存
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        if (Objects.equals(dto.getType(), 1)) {
            // 先判断成品是否能锁定
            if (inventoryFinishedService.lockNoSourceStorage(orderSkuList)) {
                dto.setStockType(StatusConstant.YES);
            } else {
                lockStorage(orderSkuList);
                dto.setStockType(StatusConstant.NO);
            }
            dto.setLockStorage(StatusConstant.YES);
        }

        // 更新订单
        dto.setStatus(OrderStatusEnum.STOCK_PREPARATION.getKey());
        // 订单为委外订单时，订单分类为委外订单，订单费用修改
        if (Objects.equals(dto.getType(), 2)) {
            dto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());
            if (!Objects.equals(dto.getSource(), 1)) {
                dto.setLssueFee(BigDecimal.ZERO);
            }
            dto.setProductTotalAmount(BigDecimal.ZERO);
            dto.setDeliveryMaterialsFee(BigDecimal.ZERO);
            dto.setManagementFee(BigDecimal.ZERO);
            dto.setPackingLabor(dto.getPackingLabor().multiply(new BigDecimal(2)));
            dto.setTotalAmount(dto.getProductTotalAmount()
                    .add(dto.getCustomProcessingFee())
                    .add(dto.getLssueFee())
                    .add(dto.getDeliveryMaterialsFee())
                    .add(dto.getPackingLabor())
                    .add(dto.getPackagingMaterialCost())
                    .add(dto.getManagementFee()));
            // 清空订单快递包材
            dto.setOrderPackageBomList(null);
        } else if (Objects.equals(dto.getType(), 1)) {
            // 拥有采购角色的账号创建的自主订单为采购订单
            Long userId = dto.getCreateUser();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }
        updateById(dto);

        // 更新订单产品
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .peek(item -> {
                    // 委外订单修改金额
                    if (ObjectUtil.equals(dto.getType(), 2)) {
                        item.setUnitPrice(BigDecimal.ZERO);
                        item.setLssueFee(BigDecimal.ZERO);
                        item.setDeliveryMaterialsFee(BigDecimal.ZERO);
                        item.setManagementFee(BigDecimal.ZERO);
                        item.setPackingLabor(item.getPackingLabor().multiply(new BigDecimal(2)));
                    }
                })
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, dto.getId());

        // 更新包材
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, dto.getId());

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.getOrDefault(item.getId(), new ArrayList<>());
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee()
                    .multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(
                    orderSkuProductionCost.getMaterialCost()
                            .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                            .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                            .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.editLinked(orderSkuProductionCostList, OrderSkuProductionCost::getOrderId, dto.getId());

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.equals(dto.getStatus(), OrderStatusEnum.STOCK_PREPARATION.getKey())
                && ObjectUtil.equals(dto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            dto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.addByOrderId(dto.getId());
            update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, dto.getId()));
        }

        // 更新订单产品包装信息
        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
        if (tempOrderPackageBomList == null) {
            tempOrderPackageBomList = new ArrayList<>();
        }
        List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderPackageBom) item)
                .collect(Collectors.toList());
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, dto.getId());

        // 更新产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.editFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 更新附件
        ObsFileUtil.editFile(dto.getFileList(), dto.getId());

        // 发送mq下载临时图稿
        sendMq(dto);
    }

    /**
     * 解锁库存
     */
    private void unLockStorage(OrderInfo orderInfo) {

        // 未锁定库存跳过
        if (ObjectUtil.notEqual(orderInfo.getLockStorage(), StatusConstant.YES)) {
            return;
        }

        // 修改订单库存锁定状态为否
        orderInfo.setLockStorage(StatusConstant.NO);
        update(q -> q.eq(BaseIdPo::getId, orderInfo.getId()).set(OrderInfo::getLockStorage, StatusConstant.NO));

        // 订单sku列表
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));

        // 过滤订单
        List<Long> bomSpecIdList = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(bomSpecIdList);
        orderSkuList = orderSkuList.stream()
                .filter(item -> bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId().equals(1L))
                .collect(Collectors.toList());

        inventoryService.unlockStorage(orderSkuList);
    }

    /**
     * 主材出库
     */
    private void semiFinishedProductOutStorage(List<InOutStorageBom> inOutStorageBomList, Map<Long, BomSpecBo> bomSpecBoMap, OrderInfo orderInfo) {
        List<InOutStorageBom> semiFinishedProductInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return false;
                    }
                    return ObjectUtil.equal(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto semiFinishedProductInOutStorageDto = new InOutStorageDto();
        semiFinishedProductInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        semiFinishedProductInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        semiFinishedProductInOutStorageDto.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
        semiFinishedProductInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        semiFinishedProductInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        semiFinishedProductInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + " 主材退料入库");
        semiFinishedProductInOutStorageDto.setInOutStorageBomList(semiFinishedProductInOutStorageBomList);
        inOutStorageService.add(semiFinishedProductInOutStorageDto);
    }

    /**
     * 包材出库
     */
    private void packagingMaterialOutStorage(List<InOutStorageBom> inOutStorageBomList, Map<Long, BomSpecBo> bomSpecBoMap, OrderInfo orderInfo) {
        List<InOutStorageBom> packagingMaterialInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return true;
                    }
                    return ObjectUtil.notEqual(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto packagingMaterialInOutStorageDto = new InOutStorageDto();
        packagingMaterialInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        packagingMaterialInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        packagingMaterialInOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
        packagingMaterialInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        packagingMaterialInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        packagingMaterialInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + " 包材退料入库");
        packagingMaterialInOutStorageDto.setInOutStorageBomList(packagingMaterialInOutStorageBomList);
        inOutStorageService.add(packagingMaterialInOutStorageDto);
    }

    /**
     * 锁定库存
     */
    private void lockStorage(List<? extends OrderSku> orderSkuList) {
        List<Long> bomSpecIdList = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(bomSpecIdList);

        boolean result = inventoryService.lockStorage(orderSkuList.stream()
                .filter(item -> bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId().equals(1L))
                .collect(Collectors.toList())
        );

        if (!result) {
            throw new ServiceException("主材库存不足，无法确认订单");
        }

    }

    private void sendMq(OrderInfoDto dto) {
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        for (OrderSkuDto orderSkuDto : orderSkuList) {
            if (orderSkuDto.getArtworkLibraryId() == null || orderSkuDto.getArtworkLibraryId() == 0L) {
                continue;
            }
            TempArtworkMessage tempArtworkMessage = new TempArtworkMessage();
            tempArtworkMessage.setImgUrl(orderSkuDto.getBlueprint());
            tempArtworkMessage.setFileUrl(orderSkuDto.getProductionDocument());
            RabbitMqUtil.send(ArtworkConfig.DIRECT_EXCHANGE_NAME, ArtworkConfig.TEMP_ARTWORK_QUEUE_NAME, tempArtworkMessage);
        }
    }

}
